@@ -77,7 +77,7 @@ class RTSRoot(CFSNode):
 
     def _list_storage_objects(self):
         self._check_self()
-        for so in StorageObject.all(self.path):
+        for so in StorageObject.all():
             yield so
 
     def _list_tpgs(self):
@@ -210,7 +210,7 @@ class RTSRoot(CFSNode):
             fm_obj = FabricModule(t['fabric'])
 
             # Instantiate target
-            Target.setup(fm_obj, list(self.storage_objects), t, err_func)
+            Target.setup(fm_obj, t, err_func)
 
         return errors
 
